package com.qmth.teachcloud.report.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.report.business.bean.dto.printOpen.PaperConfig;
import com.qmth.teachcloud.report.business.bean.dto.printOpen.PaperDimension;
import com.qmth.teachcloud.report.business.entity.TBPaper;
import com.qmth.teachcloud.report.business.service.AnalyzeDataGetAndEditService;
import com.qmth.teachcloud.report.business.service.CallPrintOpenApiService;
import com.qmth.teachcloud.report.business.service.TBExamCourseService;
import com.qmth.teachcloud.report.business.service.TBPaperService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * @Description: 分析数据获取和编辑服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-06-08
 */
@Service
public class AnalyzeDataGetAndEditServiceImpl implements AnalyzeDataGetAndEditService {
    @Resource
    private CallPrintOpenApiService callPrintOpenApiService;
    @Resource
    private TBExamCourseService tbExamCourseService;
    @Resource
    private BasicCourseService basicCourseService;

    @Resource
    private TBPaperService tbPaperService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void tbPaperDispose(Long examId, String courseCode, Long schoolId) throws IOException {
        if (tbExamCourseService.verifyExamCourseCantRun(examId,schoolId,courseCode,null)){
            throw ExceptionResultEnum.ERROR.exception("课程编号为【" + courseCode + "】的课程不能进行数据更改");
        }
        String courseName = basicCourseService.findByCourseCode(courseCode, schoolId).getName();
        PaperConfig paperConfig = callPrintOpenApiService.callPaperConfig(examId,courseCode);

        BigDecimal coefficient = BigDecimal.ZERO;
        if (Objects.nonNull(paperConfig.getCoefficient())) {
            coefficient = BigDecimal.valueOf(Double.parseDouble(String.valueOf(paperConfig.getCoefficient())));
        }
        String paperType = paperConfig.getPaperType();
        List<TBPaper> oldList = tbPaperService.list(new QueryWrapper<TBPaper>().lambda()
                .eq(TBPaper::getExamId, examId).eq(TBPaper::getCourseCode, courseCode).eq(TBPaper::getPaperType, paperType));
        Long id;
        if (oldList.size() == 1) {
            id = oldList.get(0).getId();
        } else if (oldList.size() < 1) {
            id = SystemConstant.getDbUuid();
        } else {
            throw ExceptionResultEnum.ERROR.exception("表数据异常[t_b_paper]");
        }

        TBPaper tbPaper = new TBPaper();
        tbPaper.setCourseCode(courseCode);
        tbPaper.setCourseName(courseName);
        tbPaper.setPaperType(paperConfig.getPaperType());
        Long startTime = tbPaper.getStartTime();
        Long endTime = tbPaper.getEndTime();
        if (!SystemConstant.isOneNull(startTime,endTime)){
            tbPaper.setStartTime(startTime);
            tbPaper.setEndTime(endTime);
        }
        tbPaper.setCoefficient(coefficient);
        tbPaper.setTotalScore(paperConfig.getTotalScore());
        tbPaper.setPassScore(paperConfig.getPassScore());
        tbPaper.setScoreType(paperConfig.getScoreType());
        tbPaper.setId(id);
        tbPaperService.saveOrUpdate(tbPaper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void tbPaperDimensionDispose(Long examId, String courseCode, Long schoolId) throws IOException {
        if (tbExamCourseService.verifyExamCourseCantRun(examId,schoolId,courseCode,null)){
            throw ExceptionResultEnum.ERROR.exception("课程编号为【" + courseCode + "】的课程不能进行数据更改");
        }
        List<PaperDimension> paperDimensionList = callPrintOpenApiService.callPaperDimension(examId,courseCode);
        // TODO: 2022/6/8 维度新增
//        List<TBDimension> tbDimensionList = paperDimensionList.stream().flatMap(e -> )
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void tbPaperStructDispose(Long examId, String courseCode, Long schoolId) {

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void tbPaperModuleDispose(Long examId, String courseCode, Long schoolId) {

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void tbExamStudentScoreDatasourceDispose(Long examId, String courseCode, Long schoolId) {

    }
}
